(* Consider a solution that has pow multiply operations.  We can think
   of this as a series of (pow+1) add groups interspersed with the
   multiplies.  We can rearrange the adds to reduce the length of the
   solution as follows.  If there is a multiply followed by m adds, you
   can move the m adds to the left and replacing them by 1 add.  We keep
   doing that until each group of adds is of size at most m-1, except the
   first one.  So the solution amounts to a base m number, where all
   digits except the first one are between 0 and m-1.

   So say we're working with pow.  Generically the function computed
   can be written as:

           pow
   f(x) = m   *x  +  k*a

   In order for the solution to work we need f(p) >= r and f(q) <= s.
   These two inequalities give lower and upper bounds on the value of
   k.  If the range is not empty we need to find the value of k in the
   range that is the "smallest" by the measure defined in the problem.
   
   There's a simple way to do this.  Say I give you a range, and ask
   you to find the base-10 number in the range that has the smallest
   digit total.  I scan the lower and upper bounds from left to right
   and find the first place where the digits differ.  If the rest of
   the lower bound is all 0, then the lower bound is the answer.
   Otherwise, we step up the differing lower bound digit by one, then
   add all zeros at the end.  (Note that 8000 is preferable to 7001
   because although they both have the same length, moving the 1 to
   the left wins because A comes before M in lexicographic order.)

   So we try all appropriate powers, and see which one leads to the
   shortest solution.  That's about it, other than the fact that you
   have to worry a lot about overflows, because the numbers are up to
   10^9, which means that even adding two of them together can cause
   an over flow (in 31 bit arithmetic).
*)

let solve a m p q r s =
  let floor a b = a/b in
  let ceiling a b = if a<=0 then 0 else 1+(a-1)/b in

  let safe_mult a b = if (a*b)/b = a then a*b else 0 in
  let rec power a b = if b=0 then 1 else
    let c = power a (b/2) in 
    let c2 = safe_mult c c in
      if 2*(b/2)=b then c2 else safe_mult c2 a
  in

  let shortest_in_range pow lb ub = 
  (* This returns the base m number (in a list, high order digit
     first) of pow+1 digits. The number returned has the smallest
     digit-sum from among all the numbers in the range [lb,ub]. *)

    let rec express l n accum = 
      if l=0 then n::accum else express (l-1) (n/m) ((n mod m)::accum)
    in

    let rec loop na nb accum = 
      match (na,nb) with ([],_) | (_,[]) -> List.rev accum
	| (a::ta, b::tb) -> if a=b then loop ta tb (a::accum) else begin
	    (* we should have a<b *)
	    if List.for_all (fun x -> x=0) ta then (List.rev accum)@na else
	      (List.rev accum)@[a+1]@(List.map (fun x -> 0) ta)
	  end
    in
    let rec format dl accum = 
      match dl with [] -> List.rev accum (* should not happen *)
	| d::[] -> List.rev ((d,'A')::accum)
	| d::t -> format t ((1,'M')::((d,'A')::accum))
    in
      format (loop (express pow lb []) (express pow ub []) []) []
  in    

  let solvem () = 
    (* try all feasible powers, and take the best solution *)
    let rec loop pow best bestlen = 
      let mp = power m pow in
	if mp=0 || (q*mp)/mp != q || q*mp > s then best else
	  let ub = floor (s-q*mp) a in
	  let lb = max (ceiling (r-p*mp) a) 0 in
	    if lb > ub then loop (pow+1) best bestlen else begin
	      let soln = shortest_in_range pow lb ub in
	      let len = List.fold_left (fun c (l,_) -> c+l) 0 soln in
		if best=[] || len < bestlen then 
		  loop (pow+1) soln len 
		else 
		  loop (pow+1) best bestlen
	    end
    in
      loop 0 [] 0
  in

    if p > s || s-r < q-p then (false,[]) else
    if m=1 then
      let k = ceiling (r-p) a in
	if q+a*k > s then (false,[]) else (true,[(k,'A')])
    else
      let soln = solvem () in
	if soln = [] then (false,[]) else (true,soln)

let read_int () = 
  Scanf.bscanf Scanf.Scanning.stdib " %d " (fun x -> x)

let normalize a =
  let rec loop a b = 
    match (a,b) with ([],_) -> List.rev b
    | ((0,'A')::t,_) -> loop t b
    | ((c,'M')::ta,(d,'M')::tb) -> loop ta ((c+d,'M')::tb)
    | (x::ta,_) -> loop ta (x::b)
  in loop a []

let rec solve_rest case = 
  let a = read_int() in
  let m = read_int() in
  let p = read_int() in
  let q = read_int() in
  let r = read_int() in
  let s = read_int() in
    if a>0 then begin
      Printf.printf "Case %d: " case;
      begin
	match solve a m p q r s with 
	  | (false,_) ->
	      Printf.printf "impossible\n"
	  | (true,[]) | (true,[(0,_)])-> 
	      Printf.printf "empty\n"
	  | (true,answer) ->
	      List.iter (fun (l,c) -> Printf.printf "%d%c " l c) (normalize answer);
	      Printf.printf "\n"
      end;
      Printf.printf "%!";
      solve_rest (case+1)
    end
;;

solve_rest 1
