(* It's just a matter of trying all 80 rotations, and for each one
   trying all 6 permutations of the first three points to be mapped to
   the second three.  Now we can separate the problem into the two
   dimensions.  We assume WLOG that the rescaling happens first, then the
   translation.  So we need a single transformation of the form a*x+b
   that satisfies the three pairs.  See the code below in mapcount and cp
   for an elegant way to do this.

   The following is the rotation function.  The rotation is specified
   by two points p and q, which are rotated to the x-axis.  The input
   point to be rotated is (x,y).  The output is the pair (x',y') the
   rounded result of the rotation.

   Let r = sqrt(p^2 + q^2).  The rotation matrix is:

   [ p/r   q/r]
   [-q/r   p/r]
*)

let floatme (a,b) = (float_of_int a, float_of_int b)
let round x = int_of_float (floor (x +. 0.5))

let rotate (p,q) (x,y) = 
  let (p,q) = floatme (p,q) in
  let (x,y) = floatme (x,y) in
  let r = sqrt (p *. p +. q *. q) in
  let f2x = p *. x +. q *. y in
  let f2y = -.q *. x +. p *. y in
    (round (f2x/.r), round (f2y/.r))

let mapcount x1 m1 x2 m2 x3 m3 =
  let pair x1 m1 x2 m2 = 
    let n = m1-m2 in
    let d = x1-x2 in
      if d=0 then (
	if n=0 then (0,2) else (0,0)
      ) else (
	if n mod d = 0 && n != 0 then (n/d,1) else (0,0)
      )
  in
  let comb (a1,c1) (a2,c2) = 
    match (c1,c2) with
      | (0,_) | (_,0) -> (0,0)
      | (1,1) -> if a1=a2 then (a1,1) else (0,0)
      | (_,1) -> (a2,c2)
      | (1,_) -> (a1,c1)
      | (_,_) -> (0,2)
  in
    comb (comb (0,2) (pair x1 m1 x2 m2)) (pair x2 m2 x3 m3)

let cp rot (x1,y1) (x2,y2) (x3,y3) (x4,y4) (x5,y5) (x6,y6) =
  let (x1,y1) = rotate rot (x1,y1) in
  let (x2,y2) = rotate rot (x2,y2) in
  let (x3,y3) = rotate rot (x3,y3) in
  let (_,cx) = mapcount x1 x4 x2 x5 x3 x6 in
  let (_,cy) = mapcount y1 y4 y2 y5 y3 y6 in
    cx * cy

let solve p1 p2 p3 p4 p5 p6 =
  let count = ref 0 in
  let modcount r = 
    count := !count + 
      (cp r p1 p2 p3 p4 p5 p6) +
      (cp r p1 p2 p3 p4 p6 p5) +
      (cp r p1 p2 p3 p5 p4 p6) +
      (cp r p1 p2 p3 p5 p6 p4) +
      (cp r p1 p2 p3 p6 p4 p5) +
      (cp r p1 p2 p3 p6 p5 p4)
  in
    for i = -10 to 10 do modcount (10,i) done;
    for j =  -9 to  9 do modcount (j,10) done;
    !count

let read_int () = 
  Scanf.bscanf Scanf.Scanning.stdib " %d " (fun x -> x)

let read_pair () =
  let x = read_int() in
  let y = read_int() in
    (x,y)

let rec solve_rest case = 
  let p1 = read_pair () in
  let p2 = read_pair () in
  let p3 = read_pair () in
    if (p1,p2,p3)=((0,0),(0,0),(0,0)) then () else
      let p4 = read_pair () in
      let p5 = read_pair () in
      let p6 = read_pair () in
	Printf.printf "Case %d: " case;
	begin
	  match solve p1 p2 p3 p4 p5 p6 with
	    | 0 -> Printf.printf "no solution\n"
	    | 1 -> Printf.printf "equivalent solutions\n"
	    | _ -> Printf.printf "inconsistent solutions\n"
	end;
	solve_rest (case+1)
;;

solve_rest 1
