(*
  You can do it by running a min-cost circulation problem on a
  bi-partite graph.  (The cost per unit flow on the widget edges is -1,
  so it maximizes the number of widgets.)  After each flow calculation,
  you take C, the total number of components used in this solution, and
  you compute
  
  K =  floor(C * A/B)
  
  Then you modify the flow graph by adding the additional constraint
  that each row/column as at most K components in it.  The you resolve
  for the min cost flow.  You keep repeating this until one of two
  things happens.  (1) C does not change, in which case you've reached
  an optimum solution.  or (2) the flow constraints become infeasible,
  in which case you know there is no solution.
  
  This algorithm is correct because at any point you know that C is an
  upper bound on the total number of components in any feasible
  solution.
  
  Very pessimistically, the number of iterations can be at most 1600.
  And the size of the flow graph as at most 1600 edges in it.  So all in
  all, this seems like it should work.  And there's very little to code
  beyond the min cost circulation problem.
  
  Actually, the iteration estimate is way too big.  It actually has to
  converge in at most 40 iterations.  If K does not decrease, then we've
  reached a stationary point, and K is at most 40.
*)

let inf = 1_000_000

(* Our representation is just a capacity matrix.  It's 3nn+2 square.
   The +2 is for the source and sink vertices used in the max flow.
   Vertices 0 to nn-1 are the left side.  And (nn+i, 2nn+i) are 
   a pair for each on the right.  We have to split it in two because
   the left->right edge has to be different from the right-->left edge.
   We also return a cost matrix, where cost.(i).(j) is the cost of
   pushing a unit of flow from i to j. The cost is -1 for each widget
   used, so we're going to be trying to minimize this cost.
*)
let build_graph nn input k = 
  let n = 3*nn + 2 in
  let (s,t) = (n-2,n-1) in
  let c = Array.make_matrix n n 0 in
  let cost = Array.make_matrix (n-2) (n-2) 0 in
  let lbcount = ref 0 in
    for i=0 to nn-1 do
      c.(nn+i).(2*nn+i) <- inf;
      c.(2*nn+i).(nn+i) <- inf;
      c.(i+2*nn).(i) <- k;
      for j=0 to nn-1 do
	match input.(i).[j] with
	  | 'C' -> 
	      c.(s).(j+nn) <- c.(s).(j+nn) + 1;
	      c.(i).(t) <- c.(i).(t) + 1;
	      lbcount := !lbcount + 1
	  | '.' -> 
	      c.(i).(j+nn) <- 1;
	      cost.(i).(j+nn) <- -1;
	      cost.(j+nn).(i) <- 1; 
	  | _ -> ()
      done
    done;
    (n, !lbcount, c, cost)

(* max_flow computes a max flow in the graph from s to t.  The 
   capacity matrix c is updated with the resulting residual capacities.
   The max flow value is returned *)
let max_flow n s t c = 
  let augment_a_path () =
    let parent = Array.make n (-1) in
    let rec dfs p i = 
      if parent.(i) >= 0 then () else
	begin
	  parent.(i) <- p;
	  for j=0 to n-1 do
	    if c.(i).(j) > 0 then dfs i j
	  done
	end
    in
      dfs s s;
      if parent.(t) < 0 then false else 
	let rec augment_path v = 
	  if v=s then () else 
	    let p = parent.(v) in
	      c.(p).(v) <- c.(p).(v) -1;
	      c.(v).(p) <- c.(v).(p) +1;	      
	      augment_path p
	in
	  augment_path t;
	  true
  in
  let rec loop flow = 
    if augment_a_path () then loop (flow+1) else flow
  in 
    loop 0

(* c is the residual capacity of a circulation.  We're going to evolve
   this circulation by augmenting it with cycles.  You have a cost, which
   is a skew symmetric cost matrix.  cost.(i).(j) is -1 if the edge (i,j)
   corresponds to one of the places where we might put a widget.  So if
   (i,j) has cost -1 and c.(i).(j)=0 (the residual capacity=0) then we
   know that we've used that widget slot.  So we augment the circulation
   by cycles with negative cost until this is impossible.  At this point
   we've found the optimum solution, and we can read off which widget
   slots have been used.  (Actually all we need is how many there are)

   The search for a negative cost cycle is done by the Bellman-Ford algorithm.
*)
let best_circulation n c cost = 
  let augment_some_cycles () =
    let dist = Array.make n 0 in
    let parent = Array.make n (-1) in
    let onq = Array.make n true in

    let rec pass old_queue new_queue = 
      match old_queue with [] -> List.rev new_queue
	| i::tail ->
	    let rec relax_neighbors j accum = 
	      if j=n then accum 
	      else if j=i || c.(i).(j) = 0 then relax_neighbors (j+1) accum
	      else if dist.(j) <= dist.(i) + cost.(i).(j) then relax_neighbors (j+1) accum
	      else (* now relax the edge *) (
		dist.(j) <- dist.(i) + cost.(i).(j);
		parent.(j) <- i;
		if onq.(j) then relax_neighbors (j+1) accum else (
		  onq.(j) <- true;
		  relax_neighbors (j+1) (j::accum)
		)
	      )
	    in
	      onq.(i) <- false;
	      pass tail (relax_neighbors 0 new_queue)
    in

    let cycle_check () = 
      let mark = Array.make n n in
      let rec dfs level v = 
	if v = -1 || mark.(v)<level then false
	else if mark.(v) = level then true
	else (mark.(v) <- level; dfs level parent.(v))
      in
      let rec loop level = 
	if level=n then false
	else if dfs level level then true
	else loop (level+1)
      in
	loop 0
    in

    let is_power_of_2 k = k land (k-1) = 0 in

    (* The cycle_check() for the iterations that are powers of 2 is
       an optimization (the "it=n" test is sufficient for a correct 
       algorithm).  Using powers of 2 means that the algorithm will
       use at most twice as many iterations to find a cycle as the
       optimum, which would have to look for cycles after each iteration.
    *)
    let rec passes queue it =
      if queue = [] then false            (* there is no cycle *)
      else if it = n || (is_power_of_2 it && cycle_check ()) then true  (* there is a cycle *)
      else passes (pass queue []) (it+1) 
    in
    let rec all_vertices i = if i=n then [] else i::(all_vertices (i+1)) in
      if not (passes (all_vertices 0) 0) then false else
	let rec augment_cycle v stop = 
	  let p = parent.(v) in
	    c.(p).(v) <- c.(p).(v) - 1;
	    c.(v).(p) <- c.(v).(p) + 1;
	    if p = stop then () else augment_cycle p stop
	in

	(* This DFS finds all cycles, and pushes flow around them. *)
	let mark = Array.make n n in
	let rec dfs level v = 
	  if v = -1 || mark.(v)<level then ()
	  else if mark.(v) = level then augment_cycle v v
	  else (mark.(v) <- level; dfs level parent.(v))
	in
	  for level=0 to n-1 do 
	    dfs level level
	  done;
	  true
  in
    while augment_some_cycles () do () done;
    let count = ref 0 in
      for i=0 to n-1 do for j=0 to n-1 do
	if cost.(i).(j) = -1 && c.(i).(j) = 0 then count := !count +1
      done done;
      !count
   
let solve nn aa bb input = 
  let rec loop k = 
    let (n,req,c,cost) = build_graph nn input k in
    let flow = max_flow n (n-2) (n-1) c in
      if flow < req then "impossible" else 
	let new_widgets = best_circulation (n-2) c cost in
	let total_components = req + new_widgets in
	let limit = (total_components * aa)/bb in
	  if k > limit then loop limit else string_of_int new_widgets
  in
    loop inf

let read_int () = 
  Scanf.bscanf Scanf.Scanning.stdib " %d " (fun x -> x)

let read_string () = 
  Scanf.bscanf Scanf.Scanning.stdib " %s " (fun x -> x)

let rec solve_rest case = 
  let nn = read_int() in
  let aa = read_int() in
  let bb = read_int() in
    if nn>0 then
      let input = Array.make nn "" in
	for i=0 to nn-1 do
	  input.(i) <- read_string()
	done;
	  Printf.printf "Case %d: %s%!\n" case (solve nn aa bb input);
	  solve_rest (case+1)
;;

solve_rest 1
