(* There are two key observations.  

   (1) If the sticks are folded into a single polygon, then the
   maximum area is obtained when they are inscribed in a circle.

   Proof: Suppose we inscribe it in a circle.  Attach the curved arc
   of the circle to the outside of each segment.  Call each
   so-constructed (curved on one side, straight on the other) polygon
   a lens.  Now the polygon area of any arrangement of the segments is
   just the same as the curved polygon (using the lenses) minus the
   area of all the lenses.  Clearly, when the lenses are there, the
   maximum area is when they form a circle.  QED.

   (2) If the optimum solution of the problem does not use some
   segment, then it does not use the longest segment.

   Proof: Suppose the contrary.  Then our solution uses the longest
   segment but there's some other shorter segment that is not used.
   Let L be the longest segment, and let S be the next segment along
   the line that is not used.  Consider the polygon in our solution
   containing L.  Clearly, since L>=S, adding S to this polygon
   increases its area.  Therefore our solution could not have been
   optimum.  QED.

   This immediately gives an O(n^2 log epsilon) algorithm.  We compare
   two options.  One that uses all segments, and one that does not use
   the longest segment (and possibly some others).  The former is
   solved by inscribing the segments into a circle (using binary
   search to solve for the radius).  The latter is solved by splitting
   the sequence in two and solving both halves recursively.
*)

let pi = 4. *. atan2 1.0 1.0 
let eps = 1e-15

let size s = List.fold_left (fun a b -> a+.b) 0. s

let angle_sum s r = List.fold_left 
  (fun x h -> x +. (2.0 *. (asin (h /. (2. *. r))))) 0. s
  
let area_sum s r = List.fold_left 
  (fun x h -> x +. (h/.2.) *. (sqrt (r*.r -. h*.h/.4.))) 0. s

let split_at_max s = 
  let rec split_at_val m s accum =  
    match s with 
      | [] -> failwith "split_at_max failed"
      | h::t -> 
	  if h=m then (List.rev accum, h, t)
	  else split_at_val m t (h::accum)
  in
    split_at_val (List.fold_left max 0.0 s) s []

let use_all s = 
  (* Return the area of the polygon using all the segments.
     Return 0.0 if impossible to use them all. *)
  
  let (front,m,back) = split_at_max s in
  let small = List.rev_append front back in

  let contains_center = angle_sum small (m/.2.) >= pi in

  let too_big r = 
    (* return true if r is too big a radius to handle these, and false otherwise *)
    let alpha = angle_sum small r in
    let beta = angle_sum [m] r in
      if contains_center then alpha+.beta < 2.*.pi else alpha > beta
  in
    
  let rec radius r1 r2 = 
    let r = (r1 +. r2)/. 2. in
      if (r2-.r1)/.r1 < eps then r else 
	if too_big r then radius r1 r else radius r r2
  in
    
  let rec find_big r = 
    if too_big r then r else find_big (2. *. r)
  in

    if size small <= m then 0.0 else
      let r = radius (m/.2.) (find_big (size s)) in
      let a1 = area_sum small r in
	if contains_center then 
	  a1 +. (area_sum [m] r) 
	else
	  a1 -. (area_sum [m] r) 	    

let rec solve s = 
  if List.length s < 3 then 0.0 else
    let (front,m,back) = split_at_max s in
      max (use_all s) ((solve front) +. (solve back))

let read_int () = 
  Scanf.bscanf Scanf.Scanning.stdib " %d " (fun x -> x)

let rec solve_rest case = 
  let n = read_int() in
    if n>0 then
      let rec loop i accum = 
	if i=n then List.rev accum else loop (i+1) ((float_of_int (read_int()))::accum)
      in
      let s = loop 0 [] in
	Printf.printf "Case %d: %15.8f\n" case (solve s);
	solve_rest (case+1)
;;

solve_rest 1
