(* This problem is easily solved once you have the bi-connected
   components of an undirected graph.  You partition the edges into
   bi-connected components.  The components form a tree with articulation
   vertices connecting them together.  Only the leaf nodes of this tree
   require an escape hole -- any internal component will allow escape to
   one of the neighbors no matter what vertex in the component is
   removed.

   So we just have to make sure to choose a non-articulation vertex in
   each leaf component.  The number of ways to do this is just the
   product of the leaf component sizes minus one.

   Actually if there's just one component, things are a bit different.
   In this case any pair of vertices works, so you have to return 
   (n choose 2) instead.
*)

module Iset = Set.Make (struct type t = int let compare = compare end)

let solve m edges =
  let rec max_vertex n e = match e with [] -> n
    | (x,y)::t -> max_vertex (max (max x y) n) t
  in
  let n = 1+max_vertex (-1) edges in
  let numa = Array.make n (-1) in   (* the dfs numbers *)
  let lowa = Array.make n (-1) in   (* lowpoint numbers *)
  let adj = Array.make n [] in
  let rec build_adj = function [] -> () 
    | (x,y)::t -> 
	adj.(x) <- y::(adj.(x));
	adj.(y) <- x::(adj.(y));
	build_adj t
  in
  let _ = build_adj edges in

  (* The following dfs partitions the tree edges into 
     the bi-connected components of the graph.  It returns 
     a list of components, each one is represented as a list of
     the tree edges in that component. *)

  let rec dfs v num stack accum = 
    let rec output_component e ac stack accum =
      match stack with [] -> invalid_arg "output_component failed"
	| h::t -> if h=e then (t,(h::ac)::accum) else output_component e (h::ac) t accum
    in

    let rec loop remaining_edges num low stack accum = 
      match remaining_edges with 
	| [] -> 
	    lowa.(v) <- low;
	    (num,low,stack,accum) 
	| w::t -> 
	    if numa.(w) < 0 then (* tree edge *)
	      let (num, loww, stack, accum) = dfs w num ((v,w)::stack) accum in
		if loww = numa.(v) then
		  let (stack,accum) = output_component (v,w) [] stack accum in
		    loop t num low stack accum
		else
		  loop t num (min low loww) stack accum
	    else if numa.(w) > numa.(v) then (* descendant edge *)
	      loop t num low stack accum  
	    else (* ancestor edge *)
	      loop t num (min numa.(w) low) stack accum		
    in
      numa.(v) <- num;
      loop adj.(v) (num+1) numa.(v) stack accum
  in
  let (_,_,_,edge_comps) = dfs 0 0 [] [] in
    
  let edge_list_to_set el = 
    List.fold_left (fun set (v,w) -> (Iset.add v (Iset.add w set))) Iset.empty el
  in
  let vertex_comps = List.rev_map edge_list_to_set edge_comps in
    if List.length vertex_comps = 1 then 
      let nn = Int64.of_int (Iset.cardinal (List.hd vertex_comps)) in
	(2, Int64.shift_right (Int64.mul nn (Int64.pred nn)) 1)
    else 
      let count = Array.make n 0 in
      let _ = List.iter (fun set -> Iset.iter (fun v -> count.(v) <- count.(v) +1) set) vertex_comps in
	(* now the articulation points have a count > 1. *)
      let count_articulation_vertices set = 
	Iset.fold (fun v c -> if count.(v) > 1 then c+1 else c) set 0
      in
	List.fold_left 
	  (fun (sum,prod) set -> if (count_articulation_vertices set) = 1
	   then ((sum+1),(Int64.mul prod (Int64.of_int ((Iset.cardinal set)-1))))
	   else (sum,prod))
	  (0,Int64.one) vertex_comps
      
let read_int () = 
  Scanf.bscanf Scanf.Scanning.stdib " %d " (fun x -> x)

let rec solve_rest case = 
  let m = read_int() in
    if m>0 then
      let rec loope i accum = 
	if i=m then accum else 
	  let x = read_int() in
	  let y = read_int() in
	    loope (i+1) ((x-1,y-1)::accum)
      in
      let edges = loope 0 [] in
      let (k, count) = solve m edges in
	Printf.printf "Case %d: %d %s\n" case k (Int64.to_string count);
	solve_rest (case+1)
;;

solve_rest 1
