(* Lemma: If the human follows a path that ends inside the square
   of cells reachable by a mummy, the mummy will have caught the human.

   Proof: The mummy's behavior is to move toward the human in both x
   and y.  Therefore the problem factors into the x and y coordinates.
   The mummy catches the human if and only if it catches him in both
   of the 1-d games.  It's obvious in the 1-d game that the mummy
   always moves toward the human, and it will catch him if and only if
   the human is within the radius that the mummy can reach.  QED

   Lemma: The human is caught at time t if and only if the union of
   the squares reachable by the mummies covers the square reachable by
   the human.

   Proof Intuition: Suppose at time t there's an uncovered spot.  Then
   the claim is that the human could have gotten there unscathed.
   Imagine running time backwards.  The uncovered region with the
   human in it has to grow 1 in each direction.  So the human can move
   in such a way as to remain in his reachable square at time t-1.
   Inductively, he can therefore get back to the origin at time 0.

   The algorithm developed here is based on the observation that we
   can roughly partition the mummies into four groups: the lower-left,
   lower-right, upper-left and upper-right.  As time goes on the
   lower-left group of mummies form a "profile" that marches up and to
   the right.  Each of the four groups moves in one of the four
   diagonal directions.  The time when the human dies is when the four
   profiles cover the square reachable by the human.

   I say "roughly" because a mummy right below the human is in both
   the lower left and lower right profiles. (Analogous for the other
   parts.)  The human can escape forever if and only if one or more of
   the profiles is empty.

   So the lower-left and lower-right profiles can be viewed as a lower
   bound function of x.  Similarly the upper ones are an upper bound
   function of x.  When there's any space between these, the human
   lives.  Since the functions are piecewise horizontal, we only have
   to evaluate these at places where it changes.  (Actually, you need
   to do an evaluation once somewhere in each flat part as well.)
   These evalations can be done in linear time in left to right order.
   
   We binary search on the time.  If T is the answer, then there are
   log T evaluations of the survival function.  Each of these is n log
   n because of sorting, which gives O(n (log n) (log T)).  This could
   actually be improved to O((n log n) + n log T), because each
   survival query doesn't actually require sorting, just merging the
   four profile lists offset appropriately by t.  These are sorted at
   the beginning, so merging is O(n) time.
*)

let survive t ll lr ul ur =
  (* This returns true if you can survive t turns *)
  let rec collect ll m accum = 
    match ll with [] -> accum
      | (x,_)::tail -> 
	  let x = m*(x+t) in
	    if x >= -t && x <= t 
	    then collect tail m (x::accum)
	    else collect tail m accum
  in
  let rec between li accum = 
    match li with [] -> List.rev accum 
      | h::[] -> List.rev (h::accum)
      | h::(i::_ as t1) -> 
	  if h<=i-2 then between t1 (h+1::(h::accum)) else between t1 (h::accum)
  in 

  let xlist = collect ll 1 (collect lr (-1) (collect ul 1 (collect ur (-1) [-t;t]))) in
  let xlist = List.sort compare xlist in
  let xlist = between xlist [] in
  let xlist_rev = List.rev_map (fun x -> -x) xlist in
  (* xlist is now the list of absolute x coordinates that we have to test *)
    
  let rec evaluate_prof ll xlist m accum = 
    (* this function computes the height of the given profile on each of the
       x coordinates in the x list.  It's linear time because the xlist
       and the profile are sorted. *)
    match (ll,xlist) with (_,[]) -> List.rev accum
      | ((x,y)::ltail, xx::xtail) ->
	  if xx<=x+t
	  then evaluate_prof ll xtail m ((m*(y+t))::accum)  (* the answer is y *)
	  else evaluate_prof ltail xlist m accum (* move ahead in the ll list *)
      | (_,_) -> failwith "sentinel failed"
  in    
  let y_ll =           evaluate_prof ll xlist       1  []  in
  let y_lr = List.rev (evaluate_prof lr xlist_rev   1  []) in
  let y_ul =           evaluate_prof ul xlist     (-1) []  in
  let y_ur = List.rev (evaluate_prof ur xlist_rev (-1) []) in
  let lower = List.rev_map2 max y_ll y_lr in
  let upper = List.rev_map2 min y_ul y_ur in
    List.exists2 (fun y0 y1 -> y0 <= y1-2) lower upper

let solve n input = 
  let inf = 10_000_000 in
  (* cp = create profile, which sorts by x and eliminates dominated points *)
  let cp ll =
    let rec add_to_profile p (x,y) = 
	match p with
	  | [] -> [(x,y)] 
	  | (xp,yp)::tail ->
	      if y >= yp then add_to_profile tail (x,y) else (x,y)::p
    in
    let fix (x,y) = if x=0 then (inf,y) else if y=0 then (x,inf) else (x,y) in
    let ll = List.rev_map fix ll in
    let ll = (inf,-inf)::ll in
      List.rev (List.fold_left add_to_profile [] (List.sort compare ll))
  in
  let rec separate_input list ul ur ll lr = 
    match list with [] -> (ll,lr,ul,ur)
      | (x,y)::tail -> 
	  let ll = if x<=0 && y<=0 then (x,y)::ll else ll in
	  let lr = if x>=0 && y<=0 then (-x,y)::lr else lr in
	  let ul = if x<=0 && y>=0 then (x,-y)::ul else ul in
	  let ur = if x>=0 && y>=0 then (-x,-y)::ur else ur in
	    separate_input tail ul ur ll lr
  in
  let (ll,lr,ul,ur) = separate_input input [] [] [] [] in
    if ul=[] || ur=[] || ll=[] || lr=[] then "never" else
      let (ll,lr,ul,ur) = (cp ll, cp lr, cp ul, cp ur) in

      let rec binary_search lo hi = 
	(* we know that we can survive lo but not hi.  Find the 
	   minimum which we cannot survive *)
	if lo+1 >= hi then hi else
	  let m = (lo+hi)/2 in
	  if survive m ll lr ul ur then 
	    binary_search m hi
	  else
	    binary_search lo m
      in
      let rec upper_bound t = 
	if survive t ll lr ul ur then upper_bound (2*t) else t
      in
      let answer = binary_search 0 (upper_bound 1) in
	string_of_int answer

let read_int () = 
  Scanf.bscanf Scanf.Scanning.stdib " %d " (fun x -> x)

let rec solve_rest case = 
  let n = read_int() in
    if n>=0 then
      let rec read_input i accum = 
	if i=n then accum else
	  let x = read_int() in
	  let y = read_int() in
	    read_input (i+1) ((x,y)::accum)
      in 
	Printf.printf "Case %d: %s\n%!" case (solve n (read_input 0 []));
	solve_rest (case+1)
;;

solve_rest 1
