(* There's a simple dynamic programming solution.  You insert the
   machines from lowest day to highest day.  When you insert a new
   machine you figure out the maximum amount of money you could have when
   you inserted it.  The way you do this is to consider all the prior
   machines, and for each one you see how much it would have earned at
   the current time.  You take the option that maximizes this and insert
   the new machine.
   
   The problem with this solution is that it's O(n^2).  To make this
   more efficient, you maintain a list of the machines (in order of
   increasing slope) that can possibly be optimum at some point in the
   future.  This is like maintaining the lower hull of a set of lines.
   I call this the "curve" in the comments below.
   
   The two operations we need to do on this data structure are (1)
   lookup the value of this lower hull curve at a particular point
   (before we insert a new machine), and (2) when we insert the
   machine, this new line needs to be inserted into the lower hull.
   
   The data structure is an ordered set of lines, each of which is a
   three tupple (sl,x,y), where sl is the slope of the line, and x,y
   are a point that the line goes through.  We store them in a binary
   search tree (ocaml set).  Operation (1) can be done by trying lines
   in order from left to right, until the y value decreases.  At this
   point we've gone too far.  Each line that we try that is not as
   good as the next one will be dominated everywhere to the right, and
   can be deleted from the data structure.  So the total time for (1)
   is O(n log n).
   
   Operation (2) is done by splitting the list at the slope of the new
   line to be inserted.  And then the new line may dominate a bunch of
   lines on either side of of its slope.  So we split, and traverse
   these two lists deleting as long as necessary, then we join the two
   lists together, with the new one in the middle.  Again, the total
   cost of this is O(n log n) (Actually, in the code below I didn't
   use join (known as union in ocaml).  The code turns out nicer if we
   do it based on pred and succ.)  *)

module Lset = Set.Make (struct type t = int*int*int let compare = compare end)
open Lset

let pred x s = 
  let (l,_,_) = split x s in
    if is_empty l then None else Some (max_elt l)

let succ x s = 
  let (_,_,r) = split x s in
    if is_empty r then None else Some (min_elt r)

let eval (sl,x0,y0) x = (x-x0)*sl + y0
  (* evaluate the given line at the given point *)

let xint (s1,x1,y1) (s3,x3,y3) = 
  (* Given two non-parallel lines, compute the x coorinate of their
   * intersection (in floating point) 
   *)
  (float_of_int (s1*x1 - s3*x3 + y3 - y1)) /. (float_of_int (s1 - s3))

let rec delete_to_max s x = 
  (* This evaluates the given curve at the point x.  It returns the
   *  pair (s',y) where y is the value, and s' is the new curve with the
   *  initial part that is "to the left" of x removed. 
   *)
  let m = min_elt s in
  let mval = eval m x in
  let s1 = remove m s in
    if is_empty s1 then (s,mval) else
      if eval (min_elt s1) x < mval then (s,mval) else delete_to_max s1 x

let add_new_line t s =  
  (* Adds a new line to the given curve.  Returns the new curve. *)
  let rec scan_to_left s =  
    match pred t s with None -> s | Some p ->  
      match pred p s with None -> s | Some q ->  
        if xint p t <= xint q t then scan_to_left (remove p s) else s 
  in 
  let rec scan_to_right s =  
    match succ t s with None -> s | Some p ->  
      match succ p s with None -> s | Some q ->  
        if xint p t >= xint q t then scan_to_right (remove p s) else s 
  in 
  let (tsl,_,_) = t in 
    match pred (tsl,max_int,max_int) s with  
        None -> add t (scan_to_right s) 
      | Some ((psl,_,_) as p) -> 
          if psl = tsl then 
	    if eval t 0 <= eval p 0 then s else  
              add t (scan_to_right (scan_to_left (remove p s))) 
          else 
            let s = scan_to_right (scan_to_left s) in 
              match (pred t s, succ t s) with 
                  (Some p, Some q) -> if xint p t < xint q t then add t s else s
                | _ -> add t s 
      
let solve cc dd mach = 
  let s = List.fold_left 
    (fun s (d,p,r,g) -> 
       let (s,y) = delete_to_max s d in
	 if y < p then s else add_new_line (g,d+1,y-p+r) s) 
    (singleton (0, 0, cc))
    (List.sort Pervasives.compare mach)
  in
  let (s,y) = delete_to_max s (dd+1) in
    y
  
let read_int () = 
  Scanf.bscanf Scanf.Scanning.stdib " %d " (fun x -> x)

let rec solve_rest case = 
  let nn = read_int() in
  let cc = read_int() in
  let dd = read_int() in
    if nn>0 then
      let rec loop i accum = 
	if i=nn then accum else begin
	  let d = read_int() in
	  let p = read_int() in
	  let r = read_int() in
	  let g = read_int() in
	    loop (i+1) ((d,p,r,g)::accum)
	end
      in
      let mach = loop 0 [] in
	Printf.printf "Case %d: %d\n" case (solve cc dd mach);
	solve_rest (case+1)
;;

solve_rest 1
